<?php

if (!defined('_FULLTRACE_CLASS')) {
	define('_FULLTRACE_CLASS', true);

	include('/usr/local/www/berogui/misc/ini_to_db_functions.php');
	include('/usr/local/www/berogui/includes/isgwtelnet.php');
	include('/usr/local/www/berogui/includes/Helper.php');

	class fulltrace {

		private $_ba;
		private $_isgw;

		function __construct () {
			$this->_ba = new beroAri();
			$this->_isgw = new isgwtelnet();
			$this->_isgw->isgw_login();
		}

		// get SIP UDP-Port
		private function _sip_udp () {
			$res = $this->_ba->select('SELECT bindport FROM sip_general');
			return(sqlite_fetch_single($res));
		}

		// generic data-to-logfile-function
		private function _log ($log_name, $content, $mode) {
			if (($fp = fopen('/var/log/' . $log_name, $mode)) == null) {
				return(false);
			}
			fwrite($fp, $content);
			fclose($fp);

			return(true);
		}

		// write data to log-file
		private function _log_write ($log_name, $content) {
			return($this->_log($log_name, $content, 'w'));
		}

		// append data to log-file
		private function _log_append ($log_name, $content) {
			return($this->_log($log_name, $content, 'w+'));
		}

		// start a fulltrace
		function start ($enable, $isdn_ports, $limit) {

			$this->_ba->set('trace', 'yes');

			if ($limit != null) {
				$limit_mISDN =		(count($isdn_ports) ? $limit * 0.2 * 1000 / count($isdn_ports) : 0);
				$limit_tcpdump =	$limit * 0.3;
				$limit_tcpdump_lo =	$limit * 0.2;
			} else {
				$limit_mISDN =		5000;
				$limit_tcpdump =	2;
				$limit_tcpdump_lo =	2;
			}

			$this->_log_write('fulltrace-starttime.txt', date(DATE_RFC822));

			unset($ports);
			if ($isdn_ports) {
				foreach ($isdn_ports as $isdn_port) {
					$ports .= ((strlen($ports) == 0) ? '' : ',') . $isdn_port;
				}
			}

			if ((($enable['isdn'] == 1) && (strlen($isdn_ports) != 0)) || ($enable['sip_sign'] == 1)) {

				// clear /var/log/
				$logfiles = array('fulltrace-info.txt', 'ISDNdebug-*', 'tcpDump', 'tcpDump.lo', 'analog_debug', 'DebugFiles.tar.gz');
				foreach ($logfiles as $logfile) {
					system('/bin/rm -rf /var/log/' . $logfile);
				}
				unset($logfiles);

				// raise ISGW-LogLevel to max
				$this->_isgw->logactive(1);
				$this->_isgw->loglevel(8);
				$this->_isgw->logactive_message(1);
				$this->_isgw->loglevel_message(2);
				$this->_isgw->lla();

				// retrieve data from ISGW
				$isgw_ret = $this->_isgw->isgw_isdnstate();
				$isgw_ret .= $this->_isgw->isgw_actualcalls();
				$isgw_ret .= $this->_isgw->isgw_bchannels();
				$isgw_ret .= $this->_isgw->isgw_callcompact();

				// append ISGW-data to file
				$this->_log_append('isgw.info2', $isgw_ret);
				unset($isgw_ret);

				// trace of ISDN-Ports
				if (($enable['isdn'] == 1) && (strlen($isdn_ports) != 0)) {
					system('/sbin/insmod /usr/local/modules/mISDN_debugtool.ko');
					exec('/usr/local/bin/mISDNdebugtool -d -s ' . $limit_mISDN . ' -f /var/log/ISDNdebug -p ' . $ports);
				}

				// trace of Analog-Ports
				if ($enable['analog'] == 1) {
					exec('/usr/local/sbin/analog-debugtool > /dev/null &');
				}

				// trace of GSM-Ports
				if ($enable['gsm'] == 1) {
					exec('/usr/local/sbin/gsm-debugtool > /dev/null &');
				}
				
				// trace of CAS-Ports
				if ($enable['cas'] == 1) {
					$this->_isgw->caslog(1);
				}

				// trace of SIP-Sign
				if ($enable['sip_sign'] == 1) {
					if ($enable['sip_audio'] == 1) {
						exec('/usr/local/sbin/tcpdump -s0 -w /var/log/tcpDump -U -C 2 > /dev/null &');
						exec('/usr/local/sbin/tcpdump -i lo -s0 -w /var/log/tcpDump.lo -U -C ' . $limit_tcpdump_lo . ' > /dev/null &');
					} else {
						$sip_port = $this->_sip_udp();
						$sms_port = (($enable['gsm_sms'] == 1) ? 'or tcp port 25 or tcp port 110' : '');

						exec('/usr/local/sbin/tcpdump -s0 -w /var/log/tcpDump -U -C 2 port ' . $sip_port . ' ' .$sms_port . ' > /dev/null &');
						exec('/usr/local/sbin/tcpdump -i lo -s0 -w /var/log/tcpDump.lo -U -C ' . $limit_tcpdump . ' port ' .
							$sip_port . ' ' . $sms_port .' > /dev/null &');
					}
				}
			}
		}

		// stop a fulltrace
		function stop ($set_date) {

			// retrieve data from ISGW
			$isgw_ret = $this->_isgw->isgw_isdnstate();
			$isgw_ret .= $this->_isgw->isgw_actualcalls();
			$isgw_ret .= $this->_isgw->isgw_bchannels();
			$isgw_ret .= $this->_isgw->isgw_callcompact();

			// append ISGW-data to file
			$this->_log_append('isgw.info1', $isgw_ret);
			unset($isgw_ret);

			// write misc stats to log-files
			system('/usr/local/sbin/generate_info.sh /var/log/fulltrace-info.txt');
			system('cat /proc/l1_stats > /var/log/l1_stats');
			system('cat /proc/l2_stats > /var/log/l2_stats');

			// close debug-tools
			system('/bin/sync');
			system('/usr/bin/killall tcpdump');
			system('/usr/bin/killall mISDNdebugtool');
			system('/usr/bin/killall -9 analog-debugtool');
			system('/usr/bin/killall -9 gsm-debugtool');
			system('/sbin/rmmod mISDN_debugtool');
			system('/bin/sync');

			// mark trace as ended
			$this->_ba->set('trace', 'no');

			// reset ISGW-logging-settings to initial values
			$this->_isgw->logactive(0);
			$this->_isgw->loglevel(0);
			$this->_isgw->logactive_message(0);
			$this->_isgw->loglevel_message(0);
			$this->_isgw->lld();
			
			//stop CAS log
			$this->_isgw->caslog(0);

			// create tar.gz-archive
			system('/bin/rm -f /var/log/DebugFiles.tar.gz');

			unset($debug_files);

			// add ISDN-Port-Debug-Files
			if (($isdn_ports = Helper::getIsdnPorts()) != null) {
				foreach($isdn_ports as $isdn_port) {
					if (file_exists('/var/log/ISDNdebug-' . $isdn_port)) {
						$debug_files .= ((strlen($debug_files) == 0) ? ' -C /var/log/' : '') . ' ISDNdebug-' . $isdn_port;
					}
				}
			}
			unset($isdn_ports);

			$tcpdump_files = array('tcpDump', 'tcpDump.lo');
			foreach ($tcpdump_files as $tcpdump_file) {
				if (file_exists('/var/log/' . $tcpdump_file)) {
					$debug_files .= ((strlen($debug_files) == 0) ? ' -C /var/log/' : '') . ' ' . $tcpdump_file;
				}
			}
			unset($tcpdump_files);

			$isgw_files = array(glob('/var/log/isgw.err*'), glob('/var/log/isgw.log*'));
			foreach ($isgw_files as $isgw_file) {
				foreach ($isgw_file as $file) {
					$debug_files .= ((strlen($debug_files) == 0) ? ' -C /var/log/' : '') . ' ' . basename($file);
				}
			}
			unset($isgw_files);

			$misc_files = array('fulltrace-info.txt', 'debug.txt', 'isgw.info1', 'isgw.info2', 'l1_stats', 'l2_stats', 'sofia.log', 'smgw.log', 'smgw.err',
							'analog_debug.log', 'analog_debug.log.1', 'gsm_debug.log', 'gsm_debug.log.1', 'system.log', 'dmesg.log', 'dmesg.log.1',
							'isgw_cas.log','isgw_cas.log.1','cloud.log');
			foreach ($misc_files as $misc_file) {
				$debug_files .= ((strlen($debug_files) == 0) ? ' -C /var/log/' : '') . (file_exists('/var/log/' . $misc_file) ? ' ' . $misc_file : '');
			}
			unset($misc_files);

			// add repo_info and configuration-files
			$debug_files .= ((strlen($debug_files) == 0) ? ' -C /var/log/' : '') . (file_exists('/usr/local/repo_info.txt') ? ' /usr/local/repo_info.txt' : '');
			$debug_files .= ' /usr/conf';

			// finally create archive
			$archive_name = '/var/log/DebugFiles.' . (($set_date == true) ? date('Y-m-d_H-i-s') . '.' : '') . 'tar.gz';
			system('/bin/tar -czf ' . $archive_name . ' ' . $debug_files);
			system('/bin/sync');
		}
	}

} // END DEFINE _FULLTRACE_CLASS

?>
